clear
global NU
global k
global x_ind
global v_ind
global hf_ind
global h_four
global Num_drops
global Nr

%% Wave kernel information (dimensionless units)
Amp = 0.1;        % wave amplitude (dimensionless -- see paper)
dec = 1.6;        % spatial decay length (in Faraday wavelengths)
r0  = 5.4;        % domain radius (in Faraday wavelengths)

LAM = 2*pi*r0;    % dimensionless circumference (in Faraday wavelengths)

% axisymmetric kernel (free to choose this function)
g = @(r) Amp * besselj(0, 2*pi*r) .* sech(r/dec);

%% Set number of droplets and dissipation NU = 1/M (M = memory)
Num_drops = 22;   % number of droplets (integer)
NU_diff = 0.02;   % difference in NU ** below ** critical threshold
                  % (dynamics are unstable for NU < NU_c, or M > M_c)

NU_default = 1;   % default value of NU if the instability is geometric

%% Fourier representaiton of wave kernel
% *** For large domains, need more Fourier modes, e.g. 1024 ***
Nr = 512;                               % number of Fourier modes

th = (0:(Nr-1))/Nr * 2*pi;              % theta grid points
x  = th * LAM/(2*pi);                   % x grid points (needed for plot)
kF = [0:(Nr/2-1), (-Nr/2):-1]*2*pi/LAM; % wavenumbers (for LAM domain)

% wave kernel along radial cut
H_kern = g(2*r0*sin(th/2));           % in physical space
hF     = fft(H_kern);                 % in Fourier space

% threshold out high frequencies - keep all terms with abs(hF) >  TOL
TOL    = 1e-6;  % decrease TOL for greater accuracy, but slower simulations
kind   = logical(abs(hF) > TOL);
k      = kF(kind);  % wavenumbers after truncation
h_four = hF(kind);  % Fourier representation of wave kernel along arclength
NF     = sum(kind); % number of Fourier modes for computation (NF <= Nr)

%% Find instability threshold of equidistant lattice
a   = 2*pi/Num_drops;               % angular separation parameter
th0 = (1:Num_drops) * a;            % angular positions of droplets
x0  = (1:Num_drops) * LAM/Num_drops; % arc length positions of droplets

% second derivative of wave kernel at droplet positions
Hpp = 1/Nr * real(  sum(-k.^2 .* h_four .* exp(1i * x0.' * k), 2)  );

% coefficients ck (Fourier coefficients of H''(x))
c0 = sum(Hpp);
ck = @(m) cos(m * th0) * Hpp; % m = column vector, m = 1,..., Num_drops

if all( ck((1:Num_drops).')-c0 <= 0 )   % memory-driven instability (yes/no)
    Mmin = zeros(1, Num_drops);
    for m = 1:Num_drops                 % find critical threshold for each wavenumber
        Mvals = roots([c0, ck(m), 1, 1]);
        Mvreal = Mvals(imag(Mvals) == 0); % select only real roots
        ind = Mvreal > 0;
        if any(ind)
            Mmin(m) = min(Mvreal(ind)); % destabilizes at finite M
        else
            Mmin(m) = Inf;              % never destabilizes!
            disp(['Mode ' num2str(m) ' always stable'])
        end
    end
    [Mcrit,kcrit] = min(Mmin); % critical memory is minimum over wavenumbers
    disp('--- Memory-driven instability ---')
    disp(['N = ' num2str(Num_drops) ', kc = ' num2str(kcrit), ...
        ', Mc = ' num2str(Mcrit)])
else                                    % geometric instability
    Mcrit = 0;
    [~,kcrit] = min(c0 - ck((1:Num_drops).'));
    disp('--- Geometrically unstable ---')
    disp(['N = ' num2str(Num_drops) ', kc = ' num2str(kcrit), ...
        ', Mc = ' num2str(Mcrit)])
end
NU_crit = 1/Mcrit;

if Mcrit == 0   % geometric instability
    NU = NU_default; % default value for case of geometric instability
    disp(['Taking NU = ' num2str(NU)  ' (default) for simulation'])
else            % memory-driven instability
    NU = NU_crit - NU_diff;
end

%% indices for computation
% x = droplet position
% v = droplet velocity
% hf = Fourier representation of free surface elevation
x_ind = 1:Num_drops;
v_ind = x_ind(end) + x_ind;
hf_ind = v_ind(end) + (1:NF);

numvar = 2*Num_drops + NF; % total number of variables in simulation

%% Initialisation and storage
% initialise on lattice with small velocity perturbation to velocity
pert = 1e-3; % magnitude of initial perturbation (change me)
x_init = x0;
v_init = pert * cos(kcrit * a * (1:Num_drops)); % form of linear stability
hf_init = 1/NU*(h_four .* sum(exp(-1i * x0.' * k), 1)).'; % steady wave                  

Y_init = zeros(numvar, 1); % column vector of variables
Y_init(x_ind) = x_init;
Y_init(v_ind) = v_init;
Y_init(hf_ind) = hf_init;

t_max = 2 * 1000;   % number of memory times
dt = 0.05;          % dimensionless time step
t = (0:dt:t_max).'; % vector of time steps
num_t = length(t);  % number of time steps

x_mat = zeros(num_t, Num_drops); x_mat(1, :) = x_init; % positions
v_mat = zeros(num_t, Num_drops); v_mat(1, :) = v_init; % velocities
% ** we do not the store wave field as it takes up a lot of memory **

% Runge-Kutta integration
Y = Y_init;
for ii = 2:num_t
    if mod(t(ii), 100) == 0
        disp(['Simulation ' num2str(ii/num_t * 100, '%.1f') '% complete'])
    end
    Y = ODE_RK4(Y,dt);
    x_mat(ii, :) = Y(x_ind).'; % save positions
    v_mat(ii, :) = Y(v_ind).'; % save velocities
end

%% Plot results and compare to predictions of Hopf analysis
Plotter
Hopf_analysis
